package se.su.dsv.scipro.activityplan;

import lombok.Data;
import lombok.EqualsAndHashCode;
import se.su.dsv.scipro.checklist.ChecklistTemplate;
import se.su.dsv.scipro.system.DomainObject;

import javax.persistence.*;
import java.util.Comparator;

@Data
@Entity
@EqualsAndHashCode(of = {"id"}, callSuper = false)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class ActivityTemplate extends DomainObject {

	@Id
	@GeneratedValue
	private Long id;
	
	@Column(nullable=false)
	private String title;
	
	@Lob
	private String description;

	@ManyToOne
	private ActivityPlanTemplate activityPlanTemplate;

	@Column(nullable=false)
	private int daysOffset;
	
	@Enumerated(EnumType.STRING)
	private Action action = Action.NONE;
	
	@Column(nullable=false)
	private int numberInOrder = Integer.MAX_VALUE;
	
	@ManyToOne(optional=true)
	private ChecklistTemplate checklistTemplate;
	
	public ActivityTemplate(){}
	
	public ActivityTemplate(int daysOffset){
		this.daysOffset = daysOffset;
	}

    public int getDaysOffset() {
        return daysOffset;
    }

    public int getNumberInOrder() {
        return numberInOrder;
    }

	public static class CustomComparator implements Comparator<ActivityTemplate>{
		@Override
		public int compare(ActivityTemplate o1, ActivityTemplate o2) {
			return o1.numberInOrder - o2.numberInOrder;
		}
	}
}