package se.su.dsv.scipro.peer;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.util.Comparator;

@Getter
@Setter
@ToString(exclude = {"commentThread"})
@Entity
@Table(name = "comment")
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class Comment extends DomainObject {
    @Id
    @GeneratedValue
    @Setter(AccessLevel.NONE)
    private Long id;

    @ManyToOne(optional = false)
    private User creator;

    @Lob
    private String comment;

    @ManyToOne(optional = false)
    private CommentThread commentThread;

    protected Comment() {
    }

    public Comment(final User creator, final CommentThread commentThread) {
        this.creator = creator;
        this.commentThread = commentThread;
    }

    public static class CustomComparator implements Comparator<Comment>{
        @Override
        public int compare(Comment o1, Comment o2) {
            if (o1.equals(o2)) {
                return 0;
            }
            if (o1.getDateCreated() != null && o2.getDateCreated() != null) {
                if (o1.getDateCreated().equals(o2.getDateCreated())) {
                    return o1.getId().compareTo(o2.getId());
                }
                return o1.getDateCreated().compareTo(o2.getDateCreated());
            }
            if (o1.getId() != null && o2.getId() != null) {
                return o1.getId().compareTo(o2.getId());
            }
            if (o1.comment != null && o2.comment != null) {
                return o1.comment.compareTo(o2.comment);
            }
            return 0;
        }
    }
}
