package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

class FinalSeminarApprovalServiceImpl extends AbstractReviewerApprovalServiceImpl<FinalSeminarApproval> implements FinalSeminarApprovalService {

    @Inject
    FinalSeminarApprovalServiceImpl(final Provider<EntityManager> em, final FileDescriptionService fileDescriptionService, final EventBus eventBus) {
        super(em, FinalSeminarApproval.class, QFinalSeminarApproval.finalSeminarApproval, eventBus, fileDescriptionService);
    }

    @Override
    public FinalSeminarApproval requestApproval(final Project project, FileUpload thesis, final String comment) {
        final FileDescription storedThesis = fileDescriptionService.storeFile(thesis);
        final FinalSeminarApproval finalSeminarApproval = findBy(project).map(new AbstractFunction1<FinalSeminarApproval, FinalSeminarApproval>() {
            @Override
            public FinalSeminarApproval apply(final FinalSeminarApproval input) {
                input.addNewThesis(storedThesis, comment);
                return input;
            }
        }).getOrElse(new AbstractFunction0<FinalSeminarApproval>() {
            @Override
            public FinalSeminarApproval apply() {
                return newApproval(project, storedThesis, comment);
            }
        });
        final FinalSeminarApproval persisted = save(finalSeminarApproval);
        eventBus.post(new FinalSeminarApprovalRequestEvent(persisted));
        return persisted;
    }

    @Override
    protected FinalSeminarApproval newApproval(Project project, FileDescription storedThesis, String comment) {
        return new FinalSeminarApproval(project, storedThesis, comment);
    }

    @Override
    public Option<FinalSeminarApproval> findBy(final Project project) {
        return Option.apply(findOne(QFinalSeminarApproval.finalSeminarApproval.project.eq(project)));
    }

    @Override
    public FinalSeminarApproval findByProject(Project project) {
        return findOne(QFinalSeminarApproval.finalSeminarApproval.project.eq(project));
    }
}
