package se.su.dsv.scipro.reviewing;

import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.system.AbstractRepository;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.List;

class ReviewerDeadlineFollowupServiceImpl extends AbstractRepository implements ReviewerDeadlineFollowupService {
    private final ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    private final DaysService daysService;

    @Inject
    ReviewerDeadlineFollowupServiceImpl(
            final Provider<EntityManager> em,
            final ReviewerDeadlineSettingsService reviewerDeadlineSettingsService,
            final DaysService daysService) {
        super(em);
        this.reviewerDeadlineSettingsService = reviewerDeadlineSettingsService;
        this.daysService = daysService;
    }

    @Override
    public List<Pair<RoughDraftApproval, Decision>> getLateRoughDrafts() {
        final List<RoughDraftApproval> roughDraftApprovals = from(QRoughDraftApproval.roughDraftApproval)
                .list(QRoughDraftApproval.roughDraftApproval);
        return getLateDecisions(roughDraftApprovals);
    }

    private <T extends ReviewerApproval> List<Pair<T, Decision>> getLateDecisions(final List<T> roughDraftApprovals) {
        final List<Pair<T, Decision>> late = new ArrayList<>();
        for (final T roughDraftApproval : roughDraftApprovals) {
            for (final Decision lateDecision : roughDraftApproval.getLateDecisions(daysService, reviewerDeadlineSettingsService.getInstance())) {
                late.add(new Pair<>(roughDraftApproval, lateDecision));
            }
        }
        return late;
    }

    @Override
    public List<Pair<FinalSeminarApproval, Decision>> getLateFinalSeminarApprovals() {
        List<FinalSeminarApproval> finalSeminarApprovals = from(QFinalSeminarApproval.finalSeminarApproval)
                .list(QFinalSeminarApproval.finalSeminarApproval);
        return getLateDecisions(finalSeminarApprovals);
    }

}
