package se.su.dsv.scipro.finalthesis;

import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.sort.SortOrder;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateColumn;
import se.su.dsv.scipro.components.ExportableDataPanel;
import se.su.dsv.scipro.dataproviders.FilteredDataProvider;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.repository.components.FileDownloadLink;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorAllProjectsPage;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.List;

public class SupervisorFinalThesisListingPage extends AbstractSupervisorAllProjectsPage {

    @Inject
    private FinalThesisService finalThesisService;

    public SupervisorFinalThesisListingPage(PageParameters pp) {
        super(pp);
        FilteredDataProvider<FinalThesis, FinalThesisService.Filter> provider = new FilteredDataProvider<>(finalThesisService, getFilter());
        provider.setSort("dateApproved", SortOrder.DESCENDING);
        add(new ExportableDataPanel<>("table", columns(), provider));
    }

    private IModel<FinalThesisService.Filter> getFilter() {
        FinalThesisService.Filter filter = new FinalThesisService.Filter();
        filter.setStatus(FinalThesis.Status.APPROVED);
        return Model.of(filter);
    }

    private List<? extends IColumn<FinalThesis, String>> columns() {
        return Arrays.asList(
                new DateColumn<FinalThesis>(new ResourceModel("date.approved"), "dateApproved", "dateApproved"),
                new PropertyColumn<FinalThesis, String>(new ResourceModel("project.type"), "project.projectType.name", "project.projectType.name"),
                new PropertyColumn<FinalThesis, String>(new ResourceModel("project.title"), "project.title", "project.title"),
                new PropertyColumn<FinalThesis, String>(new ResourceModel("supervisor"), "project.headSupervisor.lastName", "project.headSupervisor.fullName"),
                new ThesisFileColumn()
        );
    }

    private class ThesisFileColumn extends AbstractColumn<FinalThesis, String> {
        public ThesisFileColumn() {
            super(new ResourceModel("thesis.file"));
        }

        @Override
        public void populateItem(Item<ICellPopulator<FinalThesis>> item, String id, IModel<FinalThesis> model) {
            FileDownloadLink<FileDescription> link = new FileDownloadLink<>("name", new PropertyModel<FileDescription>(model, "fileDescription"));
            link.setBody(new PropertyModel<>(model, "fileDescription.name"));
            Fragment cell = new Fragment(id, "_name", SupervisorFinalThesisListingPage.this);
            cell.add(link);
            item.add(cell);
        }
    }

}
